@@ -3,12 +3,14 @@
 #![feature(binary_heap_into_iter_sorted)]
 #![feature(associated_type_bounds)]
 #![feature(generic_associated_types)]
+#![feature(box_syntax)]
 
 extern crate approx;
 #[cfg(feature = "openblas-system")]
 extern crate blas_src;
-extern crate env_logger;
 extern crate itertools;
+#[macro_use]
+extern crate more_asserts;
 extern crate ndarray;
 extern crate ndarray_linalg;
 extern crate ndarray_stats;
@@ -38,7 +40,7 @@ cfg_if::cfg_if! {
     if #[cfg(feature = "blas_intel-mkl")] {
     } else if #[cfg(feature = "blas_openblas-system")] {
     } else {
-        compile_error!("Must enable one of \"blas_{{intel_mkl,openblas-system}}\"");
+        compile_error!(r#"Must enable one of "blas_{{intel_mkl,openblas-system}}""#);
     }
 }
 
