@@ -1,4 +1,4 @@
-#![allow(clippy::module_name_repetitions, clippy::similar_names)]
+#![allow(clippy::module_name_repetitions, clippy::similar_names, non_snake_case)]
 //! Implementation of [star sets](https://link.springer.com/chapter/10.1007/978-3-030-30942-8_39)
 //! for representing affine transformed sets
 use crate::affine::{Affine, Affine2, Affine4};
@@ -86,11 +86,13 @@ impl<D: Dimension> Star<D> {
             constrs.add_eqn(coeffs, rhs);
         } else {
             self.constraints =
-                Polytope::nonempty_new(coeffs.to_owned().insert_axis(Axis(0)), array![rhs]);
+                Polytope::nonempty_new(&coeffs.to_owned().insert_axis(Axis(0)), &array![rhs]);
         }
         self
     }
 
+    #[must_use]
+    /// # Panics
     pub fn remove_constraint(mut self, idx: usize) -> Self {
         if let Some(ref mut constrs) = self.constraints {
             constrs.remove_eqn(idx);
@@ -277,7 +279,7 @@ impl Star2 {
                 } else if let LinearSolution::Unbounded(_) = solved {
                     NNVFloat::neg_infinity()
                 } else {
-                    panic!("Solution: {:?}", solved)
+                    panic!("Solution: {:?} on star {:?}", solved, self)
                 }
             },
         )
@@ -292,7 +294,7 @@ impl Star2 {
     /// # Panics
     /// TODO: Change output type to Option<T>
     pub fn get_output_max(&self, idx: usize, input_bounds: &Bounds1) -> NNVFloat {
-        let mut eqn = self.representation.get_eqn(idx);
+        let eqn = self.representation.get_eqn(idx);
         let shift = eqn.shift()[0];
 
         self.constraints.as_ref().map_or_else(
