@@ -1,10 +1,9 @@
 #![allow(clippy::module_name_repetitions)]
-use crate::affine::Affine2;
 use crate::bounds::Bounds1;
 use crate::deeppoly::deep_poly;
-use crate::dnn::DNNIndex;
-use crate::dnn::DNNIterator;
-use crate::dnn::DNN;
+use crate::dnn::dnn::DNN;
+use crate::dnn::dnn_iter::DNNIndex;
+use crate::dnn::dnn_iter::DNNIterator;
 use crate::gaussian::GaussianDistribution;
 use crate::num::Float;
 use crate::polytope::Polytope;
@@ -21,51 +20,21 @@ use serde::{Deserialize, Serialize};
 use std::fmt::Debug;
 use truncnorm::tilting::TiltingSolution;
 
-#[derive(Debug, Clone)]
-pub enum StarNodeOp {
-    Leaf,
-    Affine(Affine2),
-    StepRelu(usize),
-    StepReluDropout((NNVFloat, usize)),
-}
-
-impl StarNodeOp {
-    /// bounds: Output bounds: Concrete bounds
-    /// affs: Input bounds: Abstract bounds in terms of inputs
-    ///
-    /// # Panics
-    pub fn apply_bounds(
-        &self,
-        bounds: &Bounds1,
-        lower_aff: &Affine2,
-        upper_aff: &Affine2,
-    ) -> (Bounds1, (Affine2, Affine2)) {
-        match self {
-            Self::Leaf => (bounds.clone(), (lower_aff.clone(), upper_aff.clone())),
-            Self::Affine(aff) => (
-                aff.signed_apply(bounds),
-                (
-                    aff.signed_compose(lower_aff, upper_aff),
-                    aff.signed_compose(upper_aff, lower_aff),
-                ),
-            ),
-            Self::StepRelu(dim) => crate::deeppoly::deep_poly_steprelu(
-                *dim,
-                bounds.clone(),
-                lower_aff.clone(),
-                upper_aff.clone(),
-            ),
-            Self::StepReluDropout(_dim) => todo!(),
-        }
-    }
-}
-
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub enum StarNodeType {
-    Leaf,
+    Interpolate {
+        child_idx: usize,
+    },
+    Leaf {
+        safe_idx: Option<usize>,
+        unsafe_idx: Option<usize>,
+    },
     Affine {
         child_idx: usize,
     },
+    Conv {
+        child_idx: usize,
+    },
     StepRelu {
         dim: usize,
         fst_child_idx: usize,
@@ -83,8 +52,17 @@ pub enum StarNodeType {
 impl StarNodeType {
     pub fn get_child_ids(&self) -> Vec<usize> {
         match self {
-            StarNodeType::Leaf => vec![],
-            StarNodeType::Affine { child_idx } => vec![*child_idx],
+            StarNodeType::Leaf {
+                safe_idx,
+                unsafe_idx,
+            } => IntoIterator::into_iter(vec![safe_idx, unsafe_idx])
+                .filter_map(|&x| x)
+                .collect(),
+            StarNodeType::Affine { child_idx }
+            | StarNodeType::Conv { child_idx }
+            | StarNodeType::Interpolate { child_idx } => {
+                vec![*child_idx]
+            }
             StarNodeType::StepRelu {
                 dim: _,
                 fst_child_idx,
@@ -127,10 +105,14 @@ pub struct StarNode<D: Dimension> {
 }
 
 impl<D: Dimension> StarNode<D> {
-    pub fn default(star: Star<D>, axis_aligned_input_bounds: Option<Bounds1>) -> Self {
+    pub fn default(
+        star: Star<D>,
+        axis_aligned_input_bounds: Option<Bounds1>,
+        initial_idx: DNNIndex,
+    ) -> Self {
         Self {
             star,
-            dnn_index: DNNIndex::default(),
+            dnn_index: initial_idx,
             star_cdf: None,
             cdf_delta: 0.,
             axis_aligned_input_bounds,
@@ -139,14 +121,6 @@ impl<D: Dimension> StarNode<D> {
         }
     }
 
-    #[must_use]
-    pub fn with_dnn_index(mut self, dnn_index: DNNIndex) -> Self {
-        self.dnn_index = dnn_index;
-        self
-    }
-}
-
-impl<D: Dimension> StarNode<D> {
     pub fn get_star(&self) -> &Star<D> {
         &self.star
     }
@@ -156,7 +130,7 @@ impl<D: Dimension> StarNode<D> {
     }
 
     pub fn try_get_cdf(&self) -> Option<NNVFloat> {
-        return self.star_cdf;
+        self.star_cdf
     }
 
     pub fn set_cdf(&mut self, val: NNVFloat) {
@@ -236,6 +210,20 @@ impl StarNode<Ix2> {
         self.star.get_representation().apply(&x.view())
     }
 
+    #[must_use]
+    pub fn get_unsafe_star(&self, safe_value: NNVFloat) -> Self {
+        let safe_star = self.star.get_safe_subset(safe_value);
+        Self {
+            star: safe_star,
+            dnn_index: self.dnn_index,
+            star_cdf: None,
+            cdf_delta: 0.,
+            axis_aligned_input_bounds: None,
+            output_bounds: None,
+            gaussian_distribution: None,
+        }
+    }
+
     #[must_use]
     pub fn get_safe_star(&self, safe_value: NNVFloat) -> Self {
         let safe_star = self.star.get_safe_subset(safe_value);
@@ -304,18 +292,8 @@ impl StarNode<Ix2> {
         if self.axis_aligned_input_bounds.is_none() {
             self.axis_aligned_input_bounds = Some(
                 self.star
-                    .calculate_output_axis_aligned_bounding_box(&outer_bounds),
+                    .calculate_output_axis_aligned_bounding_box(outer_bounds),
             );
-            /*debug_assert!(
-                self.axis_aligned_input_bounds
-                    .clone()
-                    .unwrap()
-                    .bounds_iter()
-                    .into_iter()
-                    .all(|x| (x[[0]] <= x[[1]])),
-                "Calculated bounds are flipped! {}",
-                self.axis_aligned_input_bounds.clone().unwrap()
-            );*/
         }
         self.axis_aligned_input_bounds.as_ref().unwrap()
     }
@@ -331,7 +309,8 @@ impl StarNode<Ix2> {
             trace!("get_output_bounds on star {:?}", self.star);
             let dnn_iter = DNNIterator::new(dnn, self.dnn_index);
             self.output_bounds = Some(output_fn(deep_poly(
-                self.get_axis_aligned_input_bounds(&outer_input_bounds),
+                self.get_axis_aligned_input_bounds(outer_input_bounds),
+                dnn,
                 dnn_iter,
             )));
         }
