@@ -45,7 +45,7 @@ pub trait CensoredProbStarSet<D: 'static + Dimension>: ProbStarSet<D> {
 pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
     fn get_node_output_bounds(&mut self, node_id: usize) -> (NNVFloat, NNVFloat) {
         let outer_bounds: Bounds1 = self.get_input_bounds().as_ref().cloned().unwrap();
-        let (node_mut, loc, scale, dnn) = self.get_node_mut_with_borrows(node_id);
+        let (node_mut, _loc, _scale, dnn) = self.get_node_mut_with_borrows(node_id);
         node_mut.get_output_bounds(dnn, &|x| (x.lower()[[0]], x.upper()[[0]]), &outer_bounds)
     }
 
@@ -55,19 +55,21 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
         num_samples: usize,
         rng: &mut R,
         time_limit_opt: Option<Duration>,
-    ) {
+    ) -> NNVFloat {
+        let start_time = Instant::now();
+        info!("dfs_samples START");
         let samples = {
             let without_fixed = self.sample_root_node(num_samples, rng);
             let almost_samples = if let Some(bounds) = self.get_input_bounds() {
                 let unfixed_dims = without_fixed.shape()[0]; // Assuming rank 2 here
-                let mut fixed = bounds
-                    .fixed_vals_or_zeros()
+                let fixed_opt = bounds
+                    .fixed_vals_or_none()
                     .slice_axis(Axis(0), Slice::from(..-(unfixed_dims as isize)))
                     .insert_axis(Axis(1))
                     .to_owned();
-                if fixed.iter().any(|x| *x != 0.) {
-                    let zeros: Array2<f64> = Array2::zeros((1, 1000));
-                    fixed = fixed + zeros;
+                if fixed_opt.iter().any(Option::is_some) {
+                    let zeros: Array2<f64> = Array2::zeros((1, num_samples));
+                    let mut fixed = fixed_opt.mapv(|x| x.map_or(0., |val| val)) + zeros;
                     fixed
                         .append(Axis(0), without_fixed.view())
                         .expect("Could not append!");
@@ -83,26 +85,60 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
         let activation_patterns = self.get_dnn().calculate_activation_pattern2(samples);
         // currently, tilting is propagated, so we need to initialize it for the root node
         self.get_node_gaussian_distribution(self.get_root_id());
+        let safe_value = self.get_safe_value();
+        let mut total_infeasible_cdf = 0.;
         // For each datum
         for i in 0..num_samples {
+            if let Some(time_limit) = time_limit_opt {
+                if start_time.elapsed() >= time_limit {
+                    info!("dfs_samples STOP: time out after {} samples", i);
+                    return total_infeasible_cdf;
+                }
+            }
+            let mut path = vec![];
             // Start at the root
             let mut current_node_id = self.get_root_id();
+            path.push(current_node_id);
             let mut current_node_type = self.get_node_type(current_node_id).clone();
+            let mut sample_finished = false;
             // For each ReLU layer activation pattern
             for layer_activations in &activation_patterns {
+                if sample_finished {
+                    break;
+                }
                 // Go through the Affine
                 if let StarNodeType::Affine { child_idx } = current_node_type {
                     current_node_id = child_idx;
+                    path.push(current_node_id);
                     current_node_type = self.get_node_type(current_node_id).clone();
                 }
                 // For each activation
                 for activation in layer_activations.column(i) {
-                    // Estimate output bounds and potentially stop
-                    // TODO
+                    if let Some(feasible) = self.get_node_feasibility(current_node_id) {
+                        if !feasible {
+                            sample_finished = true;
+                            break;
+                        }
+                    }
                     let current_output_bounds = self.get_node_output_bounds(current_node_id);
+                    if current_output_bounds.1 < safe_value {
+                        self.set_node_feasibility(current_node_id, true);
+                        sample_finished = true;
+                        break;
+                    } else if current_output_bounds.0 > safe_value {
+                        current_node_id = self.infeasible_reset(
+                            current_node_id,
+                            &mut path,
+                            &mut 0.,
+                            &mut total_infeasible_cdf,
+                            rng,
+                        );
+                        sample_finished = true;
+                        break;
+                    }
                     // Select a child node based on the activation
                     if let StarNodeType::StepRelu {
-                        dim,
+                        dim: _,
                         fst_child_idx,
                         snd_child_idx,
                     } = current_node_type
@@ -114,19 +150,66 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
                                 snd_child_idx.expect("Error selecting a second child!");
                         }
                         current_node_type = self.get_node_type(current_node_id).clone();
+                        path.push(current_node_id);
                     } else {
-                        panic!("Expected a ReLU layer!");
+                        panic!("Expected a ReLU layer! Found {:?}", current_node_type);
                     }
                 }
             }
+            // Leaf reached only if sample didn't exit early
+            if !sample_finished {
+                let mut current_node_type = self.get_node_type(current_node_id).clone();
+                // Go through the last Affine
+                if let StarNodeType::Affine { child_idx } = current_node_type {
+                    current_node_id = child_idx;
+                    current_node_type = self.get_node_type(current_node_id).clone();
+                }
+                match current_node_type {
+                    StarNodeType::Leaf {
+                        unsafe_idx: Some(_),
+                        ..
+                    } => continue,
+                    StarNodeType::Leaf {
+                        unsafe_idx: None,
+                        safe_idx: safe_idx_val,
+                    } => {
+                        let safe_value = self.get_safe_value();
+                        let unsafe_node =
+                            self.get_node(current_node_id).get_unsafe_star(safe_value);
+                        let unsafe_node_idx = self.add_node(unsafe_node, current_node_id);
+                        self.set_node_type(
+                            current_node_id,
+                            StarNodeType::Leaf {
+                                safe_idx: safe_idx_val,
+                                unsafe_idx: Some(unsafe_node_idx),
+                            },
+                        );
+                        path.push(unsafe_node_idx);
+                        self.infeasible_reset(
+                            current_node_id,
+                            &mut path,
+                            &mut 0.,
+                            &mut total_infeasible_cdf,
+                            rng,
+                        );
+                    }
+                    _ => panic!(
+                        "This should always be a leaf, instead it's a {:?} in {:?}",
+                        current_node_type,
+                        self.get_node_type_iter().collect::<Vec<_>>()
+                    ),
+                }
+            }
         }
+        info!("dfs_samples STOP: complete");
+        total_infeasible_cdf
     }
 
     fn get_overapproximated_infeasible_input_regions(&self) -> Vec<Bounds1> {
         let unsafe_nodes: Vec<usize> = self
             .get_feasible_iter()
             .enumerate()
-            .filter(|(_pos, x)| !matches!(x, Some(true)))
+            .filter(|(_pos, x)| matches!(x, Some(false)))
             .map(|(pos, _x)| pos)
             .collect();
         let deduped_unsafe_nodes = unsafe_nodes.iter().filter(|idx| {
@@ -144,6 +227,36 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
             .collect()
     }
 
+    fn infeasible_reset<R: Rng>(
+        &mut self,
+        x: usize,
+        path: &mut Vec<usize>,
+        path_logp: &mut NNVFloat,
+        total_infeasible_cdf: &mut NNVFloat,
+        rng: &mut R,
+    ) -> usize {
+        self.set_node_feasibility(x, false);
+        let mut infeas_cdf = self.get_node_cdf(x, rng);
+        path.drain(..).rev().for_each(|x| {
+            // check if all chilren are infeasible
+            if self
+                .get_node_child_ids(x)
+                .iter()
+                .any(|&x| !self.is_node_infeasible(x))
+            {
+                // if not all infeasible, update CDF
+                self.add_node_cdf(x, NNVFloat::neg(NNVFloat::one()) * infeas_cdf);
+            } else {
+                // if all infeasible, set this to infeasible and continue
+                self.set_node_feasibility(x, false);
+                infeas_cdf = self.get_node_cdf(x, rng);
+            }
+        });
+        *total_infeasible_cdf += infeas_cdf;
+        *path_logp = NNVFloat::zero();
+        0
+    }
+
     /// Sample from a Gaussian distribution with an upper bound on the output value
     ///
     /// This function uses a two step sampling process. It first walks the Constellation's binary
@@ -174,7 +287,7 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
         time_limit_opt: Option<Duration>,
     ) -> Option<(Vec<Array1<NNVFloat>>, NNVFloat, NNVFloat)> {
         let start_time = Instant::now();
-        let cdf_samples = self.get_cdf_samples();
+        info!("sample_safe_star START");
         let max_iters = self.get_max_accept_reject_iters();
         let stability_eps = self.get_stability_eps();
         let mut sample_heap: BinaryHeap<FstOrdTuple<Reverse<OrderedFloat<f64>>, _>> =
@@ -183,39 +296,10 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
         let mut path = vec![];
         let mut path_logp = 0.;
         let mut total_infeasible_cdf = 0.;
-        let infeasible_reset = |me: &mut Self,
-                                x: usize,
-                                path: &mut Vec<usize>,
-                                path_logp: &mut NNVFloat,
-                                total_infeasible_cdf: &mut NNVFloat,
-                                rng: &mut R|
-         -> usize {
-            info!("Infeasible reset at depth {}!", path.len());
-            me.set_node_feasibility(x, false);
-            let mut infeas_cdf = me.get_node_cdf(x, rng);
-            path.drain(..).rev().for_each(|x| {
-                // check if all chilren are infeasible
-                if me
-                    .get_node_child_ids(x)
-                    .iter()
-                    .any(|&x| !me.is_node_infeasible(x))
-                {
-                    // if not infeasible, update CDF
-                    me.add_node_cdf(x, NNVFloat::neg(NNVFloat::one()) * infeas_cdf);
-                } else {
-                    me.set_node_feasibility(x, false);
-                    infeas_cdf = me.get_node_cdf(x, rng);
-                }
-            });
-            *total_infeasible_cdf += infeas_cdf;
-            *path_logp = NNVFloat::zero();
-            0
-        };
         loop {
-            debug!("Current node: {:?}", current_node);
             // base case for feasability
             if current_node == 0 && self.is_node_infeasible(current_node) {
-                info!("No feasible value exists!");
+                info!("sample_safe_star STOP: No feasible value exists!");
                 return None;
             }
             // Try to sample and return if a valid sample is found
@@ -225,8 +309,11 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
                 .is_some());
             match safe_sample_opt {
                 Ok(safe_samples) => {
+                    if safe_samples.iter().any(|x| x.iter().any(|v| v.abs() > 10.)) {
+                        panic!("Random safe sample big!");
+                    }
                     info!(
-                        "Safe samples found less than {} at depth {}",
+                        "sample_safe_star STOP: Safe samples found less than {} at depth {}",
                         self.get_safe_value(),
                         path.len()
                     );
@@ -239,6 +326,12 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
             };
             // check feasibility of current node
             {
+                if let Some(feasible) = self.get_node_feasibility(current_node) {
+                    if !feasible {
+                        current_node = self.get_root_id();
+                        continue;
+                    }
+                }
                 // makes the assumption that bounds are on 0th dimension of output
                 let output_bounds = if current_node == 0 {
                     (NNVFloat::neg_infinity(), NNVFloat::infinity())
@@ -251,19 +344,32 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
                     .is_some());
                 if output_bounds.1 <= self.get_safe_value() {
                     // handle case where star is safe
-                    info!(
-                        "Safe sample with value at most {} at depth {} (bounds {:?})",
-                        self.get_safe_value(),
-                        path.len(),
-                        output_bounds
-                    );
                     let safe_sample = self.sample_gaussian_node(current_node, rng, num_samples);
-                    return Some((safe_sample, path_logp, total_infeasible_cdf));
+                    if !safe_sample.is_empty() {
+                        if safe_sample.iter().any(|x| x.iter().any(|v| v.abs() > 10.)) {
+                            panic!("Safe node sample big!");
+                        }
+                        info!(
+                            "sample_safe_star STOP: Safe sample with value at most {} at depth {} (bounds {:?})",
+                            self.get_safe_value(),
+                            path.len(),
+                            output_bounds
+                        );
+                        return Some((safe_sample, path_logp, total_infeasible_cdf));
+                    } else {
+                        // Best thing to do here would be to figure out how to sample despite poor conditioning
+                        current_node = self.infeasible_reset(
+                            current_node,
+                            &mut path,
+                            &mut path_logp,
+                            &mut total_infeasible_cdf,
+                            rng,
+                        );
+                    }
                 } else if output_bounds.0 > self.get_safe_value() {
                     // handle case where star is infeasible
                     self.set_node_feasibility(current_node, false);
-                    current_node = infeasible_reset(
-                        self,
+                    current_node = self.infeasible_reset(
                         current_node,
                         &mut path,
                         &mut path_logp,
@@ -271,22 +377,41 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
                         rng,
                     );
                     continue;
-                } else if let StarNodeType::Leaf = self.get_node_type(current_node) {
-                    // do procedure to select safe part
-                    info!(
-                        "Safe sample with value at most {} at leaf (bounds {:?})",
-                        self.get_safe_value(),
-                        output_bounds
-                    );
-                    let safe_sample = self.sample_gaussian_node_safe(
-                        current_node,
-                        rng,
-                        num_samples,
-                        max_iters,
-                        self.get_safe_value(),
-                        stability_eps,
-                    );
-                    return Some((safe_sample, path_logp, total_infeasible_cdf));
+                } else {
+                    match self.get_node_type(current_node) {
+                        StarNodeType::Leaf { .. } => {
+                            // do procedure to select safe part
+                            let safe_sample = self.sample_gaussian_node_safe(
+                                current_node,
+                                rng,
+                                num_samples,
+                                max_iters,
+                                self.get_safe_value(),
+                                stability_eps,
+                            );
+                            if !safe_sample.is_empty() {
+                                if safe_sample.iter().any(|x| x.iter().any(|v| v.abs() > 10.)) {
+                                    panic!("Leaf safe sample big!");
+                                }
+                                info!(
+                                    "sample_safe_star STOP: Safe sample with value at most {} at leaf (bounds {:?})",
+                                    self.get_safe_value(),
+                                    output_bounds
+                                );
+                                return Some((safe_sample, path_logp, total_infeasible_cdf));
+                            } else {
+                                // Best thing to do here would be to figure out how to sample despite poor conditioning
+                                current_node = self.infeasible_reset(
+                                    current_node,
+                                    &mut path,
+                                    &mut path_logp,
+                                    &mut total_infeasible_cdf,
+                                    rng,
+                                );
+                            }
+                        }
+                        _ => (),
+                    }
                 }
                 // otherwise, push to path and continue expanding
                 path.push(current_node);
@@ -294,12 +419,18 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
             // check timeout
             if let Some(time_limit) = time_limit_opt {
                 if start_time.elapsed() >= time_limit {
-                    info!("Unsafe sample after timeout at depth {}", path.len());
-                    let samples = sample_heap
+                    let samples: Vec<_> = sample_heap
                         .into_iter_sorted()
                         .take(num_samples)
                         .map(|pair| pair.0 .1.mapv(|x| x.0))
                         .collect();
+                    if samples.iter().any(|x| x.iter().any(|v| v.abs() > 10.)) {
+                        panic!("Timeout safe sample big!");
+                    }
+                    info!(
+                        "sample_safe_star STOP: Unsafe sample after timeout at depth {}",
+                        path.len()
+                    );
                     return Some((samples, path_logp, total_infeasible_cdf));
                 }
             }
@@ -316,8 +447,7 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
                         path_logp += logp;
                     }
                     None => {
-                        current_node = infeasible_reset(
-                            self,
+                        current_node = self.infeasible_reset(
                             current_node,
                             &mut path,
                             &mut path_logp,
@@ -337,43 +467,62 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
         rng: &mut R,
         num_samples: usize,
     ) -> Result<Vec<Array1<NNVFloat>>, Vec<(Array1<NNVFloat>, NNVFloat)>> {
-        let unsafe_sample = self.sample_gaussian_node(current_node, rng, num_samples);
-        let vals: Vec<_> = {
+        let unsafe_sample: Vec<_> = self.sample_gaussian_node(current_node, rng, num_samples);
+        let unsafe_sample_and_vals: Vec<(&Array1<NNVFloat>, f64)> = {
             let sample_iter = unsafe_sample.iter();
             let fixed_input_part: Option<Array1<NNVFloat>> = {
-                let unsafe_len = unsafe_sample[0].len();
+                let unsafe_len = unsafe_sample.get(0).map(|x| x.len());
+                if unsafe_len.is_none() {
+                    return Err(Vec::new());
+                }
+                let unsafe_len = unsafe_len.unwrap();
                 self.get_input_bounds().as_ref().map(|bounds| {
                     let fixed_bounds: Bounds1 = bounds.split_at(bounds.ndim() - unsafe_len).0;
                     let fixed_array: Array1<NNVFloat> = fixed_bounds.lower().to_owned();
                     fixed_array
                 })
             };
+            let _node = self.get_node(current_node);
+            let _bounds = self.get_input_bounds().as_ref();
             if let Some(fix_part) = fixed_input_part {
                 sample_iter
                     .zip(iter::repeat(fix_part))
-                    .map(|(unfix, fix)| concatenate(Axis(0), &[fix.view(), unfix.view()]).unwrap())
-                    .map(|x| self.get_dnn().forward1(x))
+                    .map(|(unfix, fix)| {
+                        (
+                            unfix,
+                            concatenate(Axis(0), &[fix.view(), unfix.view()]).unwrap(),
+                        )
+                    })
+                    //.filter(|(_sample, x)| bounds.map(|b| b.is_member(&x.view())).unwrap_or(true))
+                    //.filter(|(_sample, x)| node.is_input_member(&x.view()))
+                    .map(|(sample, x)| (sample, self.get_dnn().forward1(x)[[0]]))
                     .collect()
             } else {
                 sample_iter
-                    .map(|x| self.get_dnn().forward1(x.clone()))
+                    .map(|x| (x, self.get_dnn().forward1(x.clone())[[0]]))
+                    //.filter(|(sample, _out)| {
+                    //    bounds.map(|b| b.is_member(&sample.view())).unwrap_or(true)
+                    //})
+                    //.filter(|(sample, _out)| node.is_input_member(&sample.view()))
                     .collect()
             }
         };
-        let safe_subset: Vec<_> = unsafe_sample
+
+        let safe_subset: Vec<_> = unsafe_sample_and_vals
             .iter()
-            .zip(vals.iter())
-            .filter(|(_sample, out)| out[[0]] < self.get_safe_value())
+            .filter(|(_sample, out)| *out < self.get_safe_value())
             .map(|(sample, _val)| sample.to_owned())
             .collect();
         if safe_subset.is_empty() {
-            Err(unsafe_sample
+            Err(unsafe_sample_and_vals
                 .into_iter()
-                .zip(vals.iter())
-                .map(|(x, out)| (x.to_owned(), out[[0]]))
+                .map(|(x, out)| (x.to_owned(), out))
                 .collect())
         } else {
-            Ok(safe_subset)
+            Ok(safe_subset
+                .into_iter()
+                .map(|sample| sample.clone())
+                .collect())
         }
     }
 
@@ -396,10 +545,29 @@ pub trait CensoredProbStarSet2: CensoredProbStarSet<Ix2> + ProbStarSet2 {
         let current_node_type = self.get_node_type(current_node);
         match *current_node_type {
             // leaf node, which must be partially safe and partially unsafe
-            StarNodeType::Leaf => {
-                panic!();
+            StarNodeType::Leaf {
+                safe_idx: Some(safe_idx_val),
+                ..
+            } => Some((safe_idx_val, 1.)),
+            StarNodeType::Leaf {
+                safe_idx: None,
+                unsafe_idx: unsafe_idx_val,
+            } => {
+                let safe_value = self.get_safe_value();
+                let safe_node = self.get_node(current_node).get_safe_star(safe_value);
+                let safe_node_idx = self.add_node(safe_node, current_node);
+                self.set_node_type(
+                    current_node,
+                    StarNodeType::Leaf {
+                        safe_idx: Some(safe_node_idx),
+                        unsafe_idx: unsafe_idx_val,
+                    },
+                );
+                Some((safe_node_idx, 1.))
             }
-            StarNodeType::Affine { child_idx } => Some((child_idx, path_logp)),
+            StarNodeType::Affine { child_idx }
+            | StarNodeType::Conv { child_idx }
+            | StarNodeType::Interpolate { child_idx } => Some((child_idx, path_logp)),
             StarNodeType::StepRelu {
                 fst_child_idx,
                 snd_child_idx,
