@@ -1,14 +1,17 @@
 #![cfg(test)]
 use crate::affine::Affine2;
 use crate::bounds::{Bounds, Bounds1};
-use crate::dnn::Layer;
-use crate::dnn::DNN;
+use crate::dnn::dense::Dense;
+use crate::dnn::dnn::DNN;
+use crate::dnn::relu::ReLU;
 use crate::polytope::Polytope;
 use crate::star::Star2;
 use crate::starsets::Asterism;
 use crate::starsets::Constellation;
 use ndarray::Array1;
 use ndarray::Array2;
+use ndarray::Array3;
+use ndarray::Array4;
 use ndarray::ArrayView1;
 use ndarray::Axis;
 use ndarray::Ix2;
@@ -41,6 +44,20 @@ prop_compose! {
         }
 }
 
+prop_compose! {
+    pub fn array3(dim_1: usize, dim_2:usize, dim_3:usize)
+        (v in array1(dim_1*dim_2*dim_3))-> Array3<f64> {
+            v.into_shape((dim_1, dim_2, dim_3)).unwrap()
+        }
+}
+
+prop_compose! {
+    pub fn array4(dim_1: usize, dim_2:usize, dim_3:usize, dim_4:usize)
+        (v in array1(dim_1*dim_2*dim_3*dim_4))-> Array4<f64> {
+            v.into_shape((dim_1, dim_2, dim_3, dim_4)).unwrap()
+        }
+}
+
 prop_compose! {
     pub fn affine2(in_dim: usize, out_dim: usize)
         (basis in array2(out_dim, in_dim), shift in array1(out_dim)) -> Affine2 {
@@ -79,8 +96,8 @@ prop_compose! {
             let mut dnn = DNN::default();
             for aff in affines {
                     let output_dim = aff.output_dim();
-                    dnn.add_layer(Layer::new_dense(aff));
-                    dnn.add_layer(Layer::new_relu(output_dim));
+                    dnn.add_layer(Box::new(Dense::new(aff)));
+                    dnn.add_layer(Box::new(ReLU::new(output_dim)));
                 }
             dnn
         }
@@ -154,7 +171,7 @@ prop_compose! {
 prop_compose! {
     pub fn non_empty_polytope(num_dims: usize, num_constraints: usize)
         (
-            poly in polytope_including_zero(num_dims, num_constraints)
+            _poly in polytope_including_zero(num_dims, num_constraints)
                 .prop_filter("Non-zero intercepts",
                              |i| !i.rhs().iter().any(|x| *x == 0.0_f64))
         ) -> Polytope {
